/*
 * bloom.h:
 * 
 * bloom filter for NIST NSRL
 * Originally (C) August 2006, Simson L. Garfinkel
 * Released into the public domain in September 2008.
 */

#ifndef NSRL_BLOOM_H
#define NSRL_BLOOM_H

#include <string.h>
#include <openssl/hmac.h>
#include <openssl/sha.h>
#include <sys/types.h>

#ifdef __cplusplus
extern "C" {
#endif
#ifdef NEVER_DEFINED
}
#endif

typedef struct  nsrl_bloom_ {
    u_int hash_bits;            // how many bits in the input hash (128 or 160)
    u_int hash_bytes;		// hash_bits/8 ; performance optimization
    u_int M;	                // number of bits of hash to use for each bloom function (log2(vector_bytes))
    u_int k;	                // number of bloom functions to use
    u_int vector_bytes;		// length of the vector (vector_bytes*8 = 2^M)
    u_int vector_offset;	// offset of vector in file (usually 4096)
    u_char *vector;		// the bloom filter
    char *comment;
    u_int added_items;		// How many items have been added.
    int fd;				// file descriptor for mapped file
    u_int debug;			// debug level
    u_int memmapped:1;		// true if we need to unmap; otherwise we free(vector)
    u_int free_this:1;		// should we free this on clean?
    unsigned long long hits;			// stats

    /* Encryption Support; requires OpenSSL */
    u_char *key;			// hash of passphrase; 0 for no encryption
    const EVP_MD *md;
    
} nsrl_bloom;
extern int nsrl_klookup_counters[32];


/* Service functions for hexdecimal conversion: */
int nsrl_hex2bin(unsigned char *binbuf,const char *hex);
const char *nsrl_hexbuf(char *dst,int dst_len,const unsigned char *bin,int bytes,int flag);

/** nsrl_bloom_alloc(): allocates the filter memory. */
nsrl_bloom *nsrl_bloom_alloc();		// allocate a new bloom object

/** nsrl_bloom_create:
 * Create a nsrl bloom filter for either a 128-bit or a 160-bit hash.
 * If fname is specified, the file is created; otherwise the bloom
 * filter is kept in memory. M is log2(m); k is the number of functions to use.
 * Returns 0 if successful, -1 if failure.
 */

int nsrl_bloom_create(nsrl_bloom *b,const char *fname,int hash_size, int M, int k, const char *comment);

/** nsrl_bloom_open: 
 * Opens an existing bloom filter, taking the parameters from it.
 * Returns 0 if successful, -1 if failure.
 */

int nsrl_bloom_open(nsrl_bloom *b,const char *fname,int mode);	// open an existing filter

/* add and query */

void nsrl_bloom_print_info(const nsrl_bloom *b);
void nsrl_bloom_add(nsrl_bloom *b,const unsigned char *hash); 
void nsrl_bloom_addString(nsrl_bloom *b,const char *str); 
int  nsrl_bloom_query( nsrl_bloom *b,const unsigned char *hash); // 0 not present; 1 is present
int  nsrl_bloom_queryString( nsrl_bloom *b,const char *str); // 0 not present; 1 is present
double nsrl_bloom_utilization(const nsrl_bloom *b);			  // on scale 0..1

/* Sets an encryption key */
void nsrl_set_passphrase(nsrl_bloom *b,const char *passphrase);

void  nsrl_bloom_clear(nsrl_bloom *b);	// Release resources and sanitize
void  nsrl_bloom_free(nsrl_bloom *b);	// free *b


/* debug */
void nsrl_bloom_info(char *buf,int buflen,const nsrl_bloom *b);	
void nsrl_calc_histogram(const nsrl_bloom *b,u_int counts[256]);
void nsrl_print_histogram(const nsrl_bloom *b,const u_int counts[256]);


#define NSRL_HEXBUF_UPPERCASE 0x01
#define NSRL_HEXBUF_SPACE2    0x02
#define NSRL_HEXBUF_SPACE4    0x04

#ifdef NEVER_DEFINED
{
#endif
#ifdef __cplusplus
}
#endif

#ifdef __cplusplus
#include <string>
class NSRLBloom : public nsrl_bloom {
public:
    NSRLBloom(){
	this->vector = 0;
	this->fd = 0;
	this->comment = 0;
	this->key = 0;
	nsrl_bloom_clear(this);
    };
    int open(const char *fname,int mode){
	return nsrl_bloom_open(this,fname,mode);
    }
    int create(const char *fname,int hash_size, int M, int k, const char *comment){
	return nsrl_bloom_create(this,fname,hash_size,M,k,comment);
    }
    void print_info() const {
	printf("printing info...\n");
	nsrl_bloom_print_info(this);
    }

    void add(const unsigned char *hash){
	nsrl_bloom_add(this,hash);
    }
    void addString(const char *str){
	nsrl_bloom_addString(this,str);
    }
    void addString(const std::string &str){
	nsrl_bloom_addString(this,str.c_str());
    }
    bool query(const unsigned char *hash) {// true is present; false is not present
	return nsrl_bloom_query(this,hash);
    }
    bool queryString(const char *str)  {// true is present; false is not present
	return nsrl_bloom_queryString(this,str);
    }
    bool queryString(const std::string &str)  {// true is present; false is not present
	return nsrl_bloom_queryString(this,str.c_str());
    }
    double utilization() const{
	return nsrl_bloom_utilization(this);
    }
    void set_passphrase(const char *passphrase){
	nsrl_set_passphrase(this,passphrase);
    }
    unsigned long long calchits(){
	return this->hits;
    }

    ~NSRLBloom(){
	nsrl_bloom_clear(this);
    }
};
#endif
#endif
